@@ -1,20 +1,25 @@
 import sqlalchemy as sa
 from sqlalchemy import ForeignKey
+from sqlalchemy import Identity
 from sqlalchemy import Integer
+from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
-from sqlalchemy.ext import declarative as decl
-from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import class_mapper
-from sqlalchemy.orm import clear_mappers
 from sqlalchemy.orm import close_all_sessions
 from sqlalchemy.orm import configure_mappers
+from sqlalchemy.orm import declared_attr
 from sqlalchemy.orm import deferred
-from sqlalchemy.orm import mapper
+from sqlalchemy.orm import Mapped
+from sqlalchemy.orm import mapped_column
 from sqlalchemy.orm import relationship
+from sqlalchemy.orm import Session
+from sqlalchemy.orm import with_polymorphic
+from sqlalchemy.orm.decl_api import registry
 from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
+from sqlalchemy.testing import expect_raises_message
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_
 from sqlalchemy.testing import is_false
@@ -29,15 +34,17 @@ Base = None
 class DeclarativeTestBase(fixtures.TestBase, testing.AssertsExecutionResults):
     def setup_test(self):
         global Base
-        Base = decl.declarative_base(testing.db)
+        self.mapper_registry = registry()
+        Base = self.mapper_registry.generate_base()
 
     def teardown_test(self):
         close_all_sessions()
-        clear_mappers()
+        self.mapper_registry.dispose()
         Base.metadata.drop_all(testing.db)
 
 
 class DeclarativeInheritanceTest(DeclarativeTestBase):
+    @testing.emits_warning(r".*does not indicate a 'polymorphic_identity'")
     def test_we_must_copy_mapper_args(self):
         class Person(Base):
 
@@ -76,7 +83,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             }
 
         assert class_mapper(Person).version_id_col == "a"
-        assert class_mapper(Person).include_properties == set(["id", "a", "b"])
+        assert class_mapper(Person).include_properties == {"id", "a", "b"}
 
     def test_custom_join_condition(self):
         class Foo(Base):
@@ -308,7 +315,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             discriminator = Column("type", String(50))
             __mapper_args__ = {"polymorphic_on": discriminator}
 
-        class MyMixin(object):
+        class MyMixin:
 
             pass
 
@@ -324,7 +331,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
         assert class_mapper(Engineer).inherits is class_mapper(Person)
 
     def test_intermediate_abstract_class_on_classical(self):
-        class Person(object):
+        class Person:
             pass
 
         person_table = Table(
@@ -334,7 +341,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             Column("kind", String(50)),
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Person,
             person_table,
             polymorphic_on="kind",
@@ -357,7 +364,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
         eq_(set(class_mapper(Manager).class_manager), {"id", "kind"})
 
     def test_intermediate_unmapped_class_on_classical(self):
-        class Person(object):
+        class Person:
             pass
 
         person_table = Table(
@@ -367,7 +374,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             Column("kind", String(50)),
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Person,
             person_table,
             polymorphic_on="kind",
@@ -390,7 +397,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
         eq_(set(class_mapper(Manager).class_manager), {"id", "kind"})
 
     def test_class_w_invalid_multiple_bases(self):
-        class Person(object):
+        class Person:
             pass
 
         person_table = Table(
@@ -400,7 +407,7 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             Column("kind", String(50)),
         )
 
-        mapper(
+        self.mapper_registry.map_imperatively(
             Person,
             person_table,
             polymorphic_on="kind",
@@ -653,6 +660,25 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             Engineer(name="vlad", primary_language="cobol"),
         )
 
+    def test_single_cols_on_sub_base_of_subquery(self):
+        """
+        found_during_type_annotation
+
+        """
+        t = Table("t", Base.metadata, Column("id", Integer, primary_key=True))
+
+        class Person(Base):
+            __table__ = select(t).subquery()
+
+        with expect_raises_message(
+            sa.exc.ArgumentError,
+            r"Can't declare columns on single-table-inherited subclass "
+            r".*Contractor.*; superclass .*Person.* is not mapped to a Table",
+        ):
+
+            class Contractor(Person):
+                contractor_field = Column(String)
+
     def test_single_cols_on_sub_base_of_joined(self):
         """test [ticket:3895]"""
 
@@ -673,6 +699,9 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             __tablename__ = "employee"
 
             id = Column(Integer, ForeignKey(Person.id), primary_key=True)
+            __mapper_args__ = {
+                "polymorphic_identity": "employee",
+            }
 
         class Engineer(Employee):
             __mapper_args__ = {"polymorphic_identity": "engineer"}
@@ -780,11 +809,10 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             [Person.__table__.c.name, Person.__table__.c.primary_language],
         )
 
-    @testing.skip_if(
-        lambda: testing.against("oracle"),
-        "Test has an empty insert in it at the moment",
-    )
-    def test_columns_single_inheritance_conflict_resolution(self):
+    @testing.variation("decl_type", ["legacy", "use_existing_column"])
+    def test_columns_single_inheritance_conflict_resolution(
+        self, connection, decl_base, decl_type
+    ):
         """Test that a declared_attr can return the existing column and it will
         be ignored.  this allows conditional columns to be added.
 
@@ -792,18 +820,25 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
 
         """
 
-        class Person(Base):
+        class Person(decl_base):
             __tablename__ = "person"
-            id = Column(Integer, primary_key=True)
+            id = Column(Integer, Identity(), primary_key=True)
 
         class Engineer(Person):
 
             """single table inheritance"""
 
-            @declared_attr
-            def target_id(cls):
-                return cls.__table__.c.get(
-                    "target_id", Column(Integer, ForeignKey("other.id"))
+            if decl_type.legacy:
+
+                @declared_attr
+                def target_id(cls):
+                    return cls.__table__.c.get(
+                        "target_id", Column(Integer, ForeignKey("other.id"))
+                    )
+
+            elif decl_type.use_existing_column:
+                target_id: Mapped[int] = mapped_column(
+                    ForeignKey("other.id"), use_existing_column=True
                 )
 
             @declared_attr
@@ -814,19 +849,26 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
 
             """single table inheritance"""
 
-            @declared_attr
-            def target_id(cls):
-                return cls.__table__.c.get(
-                    "target_id", Column(Integer, ForeignKey("other.id"))
+            if decl_type.legacy:
+
+                @declared_attr
+                def target_id(cls):
+                    return cls.__table__.c.get(
+                        "target_id", Column(Integer, ForeignKey("other.id"))
+                    )
+
+            elif decl_type.use_existing_column:
+                target_id: Mapped[int] = mapped_column(
+                    ForeignKey("other.id"), use_existing_column=True
                 )
 
             @declared_attr
             def target(cls):
                 return relationship("Other")
 
-        class Other(Base):
+        class Other(decl_base):
             __tablename__ = "other"
-            id = Column(Integer, primary_key=True)
+            id = Column(Integer, Identity(), primary_key=True)
 
         is_(
             Engineer.target_id.property.columns[0],
@@ -836,22 +878,25 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             Manager.target_id.property.columns[0], Person.__table__.c.target_id
         )
         # do a brief round trip on this
-        Base.metadata.create_all(testing.db)
-        session = fixture_session()
-        o1, o2 = Other(), Other()
-        session.add_all(
-            [Engineer(target=o1), Manager(target=o2), Manager(target=o1)]
-        )
-        session.commit()
-        eq_(session.query(Engineer).first().target, o1)
+        decl_base.metadata.create_all(connection)
+        with Session(connection) as session:
+            o1, o2 = Other(), Other()
+            session.add_all(
+                [Engineer(target=o1), Manager(target=o2), Manager(target=o1)]
+            )
+            session.commit()
+            eq_(session.query(Engineer).first().target, o1)
 
-    def test_columns_single_inheritance_conflict_resolution_pk(self):
+    @testing.variation("decl_type", ["legacy", "use_existing_column"])
+    def test_columns_single_inheritance_conflict_resolution_pk(
+        self, decl_base, decl_type
+    ):
         """Test #2472 in terms of a primary key column.  This is
         #4352.
 
         """
 
-        class Person(Base):
+        class Person(decl_base):
             __tablename__ = "person"
             id = Column(Integer, primary_key=True)
 
@@ -861,20 +906,34 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
 
             """single table inheritance"""
 
-            @declared_attr
-            def target_id(cls):
-                return cls.__table__.c.get(
-                    "target_id", Column(Integer, primary_key=True)
+            if decl_type.legacy:
+
+                @declared_attr
+                def target_id(cls):
+                    return cls.__table__.c.get(
+                        "target_id", Column(Integer, primary_key=True)
+                    )
+
+            elif decl_type.use_existing_column:
+                target_id: Mapped[int] = mapped_column(
+                    primary_key=True, use_existing_column=True
                 )
 
         class Manager(Person):
 
             """single table inheritance"""
 
-            @declared_attr
-            def target_id(cls):
-                return cls.__table__.c.get(
-                    "target_id", Column(Integer, primary_key=True)
+            if decl_type.legacy:
+
+                @declared_attr
+                def target_id(cls):
+                    return cls.__table__.c.get(
+                        "target_id", Column(Integer, primary_key=True)
+                    )
+
+            elif decl_type.use_existing_column:
+                target_id: Mapped[int] = mapped_column(
+                    primary_key=True, use_existing_column=True
                 )
 
         is_(
@@ -885,20 +944,30 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             Manager.target_id.property.columns[0], Person.__table__.c.target_id
         )
 
-    def test_columns_single_inheritance_cascading_resolution_pk(self):
+    @testing.variation("decl_type", ["legacy", "use_existing_column"])
+    def test_columns_single_inheritance_cascading_resolution_pk(
+        self, decl_type
+    ):
         """An additional test for #4352 in terms of the requested use case."""
 
         class TestBase(Base):
             __abstract__ = True
 
-            @declared_attr.cascading
-            def id(cls):
-                col_val = None
-                if TestBase not in cls.__bases__:
-                    col_val = cls.__table__.c.get("id")
-                if col_val is None:
-                    col_val = Column(Integer, primary_key=True)
-                return col_val
+            if decl_type.legacy:
+
+                @declared_attr.cascading
+                def id(cls):  # noqa: A001
+                    col_val = None
+                    if TestBase not in cls.__bases__:
+                        col_val = cls.__table__.c.get("id")
+                    if col_val is None:
+                        col_val = Column(Integer, primary_key=True)
+                    return col_val
+
+            elif decl_type.use_existing_column:
+                id: Mapped[int] = mapped_column(  # noqa: A001
+                    primary_key=True, use_existing_column=True
+                )
 
         class Person(TestBase):
             """single table base class"""
@@ -906,10 +975,10 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             __tablename__ = "person"
 
         class Engineer(Person):
-            """ single table inheritance, no extra cols """
+            """single table inheritance, no extra cols"""
 
         class Manager(Person):
-            """ single table inheritance, no extra cols """
+            """single table inheritance, no extra cols"""
 
         is_(Engineer.id.property.columns[0], Person.__table__.c.id)
         is_(Manager.id.property.columns[0], Person.__table__.c.id)
@@ -968,10 +1037,11 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
         sess.add(c2)
         sess.flush()
         sess.expunge_all()
+
+        wp = with_polymorphic(Person, [Engineer])
         eq_(
-            sess.query(Person)
-            .with_polymorphic(Engineer)
-            .filter(Engineer.primary_language == "cobol")
+            sess.query(wp)
+            .filter(wp.Engineer.primary_language == "cobol")
             .first(),
             Engineer(name="vlad"),
         )
@@ -1006,9 +1076,15 @@ class DeclarativeInheritanceTest(DeclarativeTestBase):
             __mapper_args__ = {"polymorphic_identity": "manager"}
             id = Column(Integer, ForeignKey("people.id"), primary_key=True)
             golf_swing = Column(String(50))
+            __mapper_args__ = {
+                "polymorphic_identity": "manager",
+            }
 
         class Boss(Manager):
             boss_name = Column(String(50))
+            __mapper_args__ = {
+                "polymorphic_identity": "boss",
+            }
 
         is_(
             Boss.__mapper__.column_attrs["boss_name"].columns[0],
