@@ -48,7 +48,6 @@ from sqlalchemy.testing.util import count_cache_key_tuples
 from sqlalchemy.testing.util import total_size
 from test.orm import _fixtures
 from .inheritance import _poly_fixtures
-from .test_events import _RemoveListeners
 from .test_query import QueryTest
 
 
@@ -141,6 +140,12 @@ class CacheKeyTest(fixtures.CacheKeyFixture, _fixtures.FixtureTest):
                 with_expression(User.name, null()),
                 with_expression(User.name, func.foobar()),
                 with_expression(User.name, User.name == "test"),
+            ),
+            compare_values=True,
+        )
+
+        self._run_cache_key_fixture(
+            lambda: (
                 Load(User).with_expression(User.name, true()),
                 Load(User).with_expression(User.name, null()),
                 Load(User).with_expression(User.name, func.foobar()),
@@ -154,7 +159,7 @@ class CacheKeyTest(fixtures.CacheKeyFixture, _fixtures.FixtureTest):
 
         from sqlalchemy import Column, Integer, String
 
-        class Foo(object):
+        class Foo:
             id = Column(Integer)
             name = Column(String)
 
@@ -172,7 +177,7 @@ class CacheKeyTest(fixtures.CacheKeyFixture, _fixtures.FixtureTest):
     def test_loader_criteria_bound_param_thing(self):
         from sqlalchemy import Column, Integer
 
-        class Foo(object):
+        class Foo:
             id = Column(Integer)
 
         def go(param):
@@ -214,22 +219,18 @@ class CacheKeyTest(fixtures.CacheKeyFixture, _fixtures.FixtureTest):
             lambda: (
                 joinedload(User.addresses),
                 joinedload(User.addresses.of_type(aliased(Address))),
-                joinedload("addresses"),
                 joinedload(User.orders),
                 joinedload(User.orders.and_(Order.id != 5)),
                 joinedload(User.orders.and_(Order.id == 5)),
                 joinedload(User.orders.and_(Order.description != "somename")),
-                joinedload(User.orders).selectinload("items"),
                 joinedload(User.orders).selectinload(Order.items),
                 defer(User.id),
-                defer("id"),
                 defer("*"),
                 defer(Address.id),
                 subqueryload(User.orders),
                 selectinload(User.orders),
                 joinedload(User.addresses).defer(Address.id),
                 joinedload(aliased(User).addresses).defer(Address.id),
-                joinedload(User.addresses).defer("id"),
                 joinedload(User.orders).joinedload(Order.items),
                 joinedload(User.orders).subqueryload(Order.items),
                 subqueryload(User.orders).subqueryload(Order.items),
@@ -248,6 +249,7 @@ class CacheKeyTest(fixtures.CacheKeyFixture, _fixtures.FixtureTest):
         User, Address, Keyword, Order, Item = self.classes(
             "User", "Address", "Keyword", "Order", "Item"
         )
+        Dingaling = self.classes.Dingaling
 
         self._run_cache_key_fixture(
             lambda: (
@@ -255,7 +257,9 @@ class CacheKeyTest(fixtures.CacheKeyFixture, _fixtures.FixtureTest):
                     joinedload(Address.dingaling)
                 ),
                 joinedload(User.addresses).options(
-                    joinedload(Address.dingaling).options(load_only("name"))
+                    joinedload(Address.dingaling).options(
+                        load_only(Dingaling.id)
+                    )
                 ),
                 joinedload(User.orders).options(
                     joinedload(Order.items).options(joinedload(Item.keywords))
@@ -411,13 +415,13 @@ class CacheKeyTest(fixtures.CacheKeyFixture, _fixtures.FixtureTest):
         """test for issue discovered in #7394"""
 
         @registry.mapped
-        class User2(object):
+        class User2:
             __table__ = self.tables.users
 
             name_syn = synonym("name")
 
         @registry.mapped
-        class Address2(object):
+        class Address2:
             __table__ = self.tables.addresses
 
             name_syn = synonym("email_address")
@@ -1042,7 +1046,9 @@ class CompositeTest(fixtures.MappedTest):
         eq_(stmt._generate_cache_key(), stmt2._generate_cache_key())
 
 
-class EmbeddedSubqTest(_RemoveListeners, DeclarativeMappedTest):
+class EmbeddedSubqTest(
+    fixtures.RemoveORMEventsGlobally, DeclarativeMappedTest
+):
     """test #8790.
 
     it's expected that cache key structures will change, this test is here
@@ -1087,7 +1093,11 @@ class EmbeddedSubqTest(_RemoveListeners, DeclarativeMappedTest):
                 "concrete": True,
             }
 
-    @testing.combinations("tuples", "memory", argnames="assert_on")
+        Base.registry.configure()
+
+    @testing.combinations(
+        "tuples", ("memory", testing.requires.is64bit), argnames="assert_on"
+    )
     def test_cache_key_gen(self, assert_on):
         Employee = self.classes.Employee
 
